from flask import Blueprint, request, jsonify, make_response, json
from app.rooms.models import Room
from app.gateways.models import Gateways
from app.rooms.schemas import room_schema
from app import db
from app.utils import token_required, paginated_query, delete_record, single_record_query
from app.rooms.utills import handle_db_commit
from sqlalchemy.orm import aliased

rooms = Blueprint('rooms', __name__)

@rooms.route('/api/v1/add/room', methods=['POST'])
@token_required(1, 2, 3)
def create_room(current_user):
    try:
        room_dict = json.loads(request.get_json()['formData'])
        loaded_room_data = room_schema.load({
            'zone_id': str(room_dict['zone_id']),
            'site_id': request.args.get('site_id'),
            'room_name': room_dict['room_name'].strip(),
            'x_min': room_dict['x_min'],
            'y_min': room_dict['y_min'],
            'x_max': room_dict['x_max'],
            'y_max': room_dict['y_max'],
            'zone_name': room_dict['zone_name'].strip()
        })
        loaded_room_data.pop('zone_name')
        return handle_db_commit(Room(**loaded_room_data), 'create')
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'failed added Room', "status": "error"}), 500)

@rooms.route('/room_name', methods=['GET'])
@token_required(1, 2, 3)
def room_name(current_user):
    try:
        search = request.args.get('search')
        gateway_room_ids = [gateway.room_id for gateway in db.session.query(aliased(Gateways).room_id).all()]
        query = Room.query.filter(~Room.id.in_(gateway_room_ids)).filter_by(site_id=request.args.get('site_id'))
        if search:
            query = query.filter(Room.room_name.ilike(f'%{search}%'))
        serialized_rooms = []
        for room in query.all():
            if room.room_name is None:
                return jsonify({"error": "Room name is required"}), 400
            serialized_rooms.append({
                'id': room.id,
                'name': room.room_name,
                'site_id': room.site_id,
                'zone_id': room.zone_id
            })
        if not serialized_rooms:
            serialized_rooms = [{
                'id': "",
                'name': "Room not available",
                'site_id': "",
                'zone_id': ""
            }]
        return jsonify({"room": serialized_rooms}), 200
    except Exception as e:
        return jsonify({"error": str(e)}), 500

@rooms.route('/api/v1/get/room', methods=['GET'])
@token_required(1, 2, 3)
def get_room(current_user):
    try:
        rooms_paginated, data = paginated_query(Room.query.order_by(Room.room_name.asc()),
                                                [Room.room_name], Room.site_id)
        if rooms_paginated:
            serialized_rooms = []
            for room in rooms_paginated:
                if room.id == 1:  
                    continue
                serialized_rooms.append({
                    'id': room.id,
                    'zone_id': room.zone_id,
                    "site_id": room.site_id,
                    'zone_name': room.zone.zone_name ,
                    'room_name': room.room_name,
                    'x_min': room.x_min,
                    'y_min': room.y_min,
                    'x_max': room.x_max,
                    'y_max': room.y_max,
                })
            response_data = {'room': serialized_rooms, 'data': data}
            return make_response(jsonify(response_data), 200)
        else:
            response_data = {'message': 'No Room found', 'data': data}
            return make_response(jsonify(response_data), 200)
    except Exception as e:
        return make_response(jsonify({'message': f'Error: {e}', 'status': 'error'}), 200)

@rooms.route('/api/v1/get/room_by_id/<int:id>', methods=['GET'])
@token_required(1, 2, 3)
def get_room_by_id(current_user, id):
    try:
        room = single_record_query(Room, Room.id, id, current_user)
        if room:
            zone_name = room.zone.zone_name
            return make_response(jsonify({'room': {
                'id': room.id,
                'zone_id': room.zone_id,
                'room_name': room.room_name,
                'x_min': room.x_min,
                'y_min': room.y_min,
                'x_max': room.x_max,
                'y_max': room.y_max,
                'zone_name': zone_name
            }}), 200)
        return make_response(jsonify({'message': 'Room not found', "status": "error"}), 500)
    except Exception as e:
        return make_response(jsonify({'message': f'error: {e}', "status": "error"}), 500)

@rooms.route('/api/v1/update_room/<int:room_id>', methods=['PUT'])
@token_required(1, 2, 3)
def update_room(current_user, room_id):
    try:
        room_to_update = single_record_query(Room, Room.id, room_id, current_user)
        if not room_to_update:
            return make_response(jsonify({'message': 'Room not found', "status": "error"}), 404)
        room_dict = json.loads(request.get_json()['formData'])
        loaded_room_data = room_schema.load({
            'zone_id': str(room_dict['zone_id']),
            'site_id': request.args.get('site_id'),
            'room_name': room_dict['room_name'].strip(),
            'x_min': room_dict['x_min'],
            'y_min': room_dict['y_min'],
            'x_max': room_dict['x_max'],
            'y_max': room_dict['y_max'],
            'zone_name': room_dict['zone_name'].strip()
        })
        loaded_room_data.pop('zone_name')
        for key, value in loaded_room_data.items():
            setattr(room_to_update, key, value)
        db.session.commit()
        return make_response(jsonify({'message': 'Room updated successfully', "status": "Success"}), 200)
    except Exception as e:
        return make_response(jsonify({'args': e.args, "message": 'Failed to update Room', "status": "error"}), 500)

@rooms.route('/api/v1/delete-room/<int:id>', methods=['DELETE'])
@token_required(1, 2, 3)
def delete_room(current_user, id):
    return delete_record(Room, current_user, id, "Room", "id")